/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements. See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership. The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.cxf.jaxrs.client;

import java.net.URI;
import java.util.Map;

import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.MultivaluedMap;

import org.apache.cxf.common.util.ProxyHelper;
import org.apache.cxf.configuration.security.AuthorizationPolicy;
import org.apache.cxf.endpoint.ConduitSelector;
import org.apache.cxf.endpoint.Endpoint;
import org.apache.cxf.endpoint.UpfrontConduitSelector;
import org.apache.cxf.feature.AbstractFeature;
import org.apache.cxf.jaxrs.AbstractJAXRSFactoryBean;
import org.apache.cxf.jaxrs.JAXRSServiceFactoryBean;
import org.apache.cxf.jaxrs.JAXRSServiceImpl;
import org.apache.cxf.jaxrs.impl.MetadataMap;
import org.apache.cxf.jaxrs.model.ClassResourceInfo;
import org.apache.cxf.service.Service;

public class AnotherJAXRSClientFactoryBean extends JAXRSClientFactoryBean {

	private boolean anotherInheritHeaders;

    public AnotherJAXRSClientFactoryBean() {
        this(new JAXRSServiceFactoryBean());
    }
    
    public AnotherJAXRSClientFactoryBean(JAXRSServiceFactoryBean serviceFactory) {
        super(serviceFactory);
    }
    
    public void setInheritHeaders(boolean ih) {
        anotherInheritHeaders = ih;
        super.setInheritHeaders(ih);
    }
 
    public Client createWithValues(Object... varValues) {
        checkResources(false);
        
        try {
            Endpoint ep = createEndpoint();
            URI baseURI = URI.create(getAddress());
            ClassResourceInfo cri = serviceFactory.getClassResourceInfo().get(0);
            boolean isRoot = cri.getURITemplate() != null;
            AnotherClientProxyImpl proxyImpl = new AnotherClientProxyImpl(baseURI, baseURI, cri, isRoot, anotherInheritHeaders,
                                                            varValues);
            initClient(proxyImpl, ep);    
            
            try {
                return (Client)ProxyHelper.getProxy(cri.getServiceClass().getClassLoader(),
                                        new Class[]{cri.getServiceClass(), Client.class, 
                                                    InvocationHandlerAware.class}, 
                                        proxyImpl);
            } catch (Exception ex) {
                return (Client)ProxyHelper.getProxy(Thread.currentThread().getContextClassLoader(),
                                                    new Class[]{cri.getServiceClass(), Client.class, 
                                                                InvocationHandlerAware.class}, 
                                     proxyImpl);
            }
        } catch (Exception ex) {
            throw new WebApplicationException();
        }
    }
} 
